@@ -188,6 +188,7 @@ class ValiotWorker():
         self.jobConfigMode = JobConfigMode.KEEP
         self.context = None
         self.unsubJobCreated = None
+        self.unsubLockUpdated = None
         self.useRedis = False
         self.redis = WorkerRedis()
         self.continuousJobs = []
@@ -533,6 +534,9 @@ class ValiotWorker():
             'active': False,
             'endAt': getUtcDate(datetime.now()).strftime("%Y-%m-%dT%H:%M:%SZ")
         }
+        lock, errors = self.gql.query(queries.get_queueLock, variables={"lockId": lock['id']})
+        if not lock:
+            return True # lock already unlocked
         data, errors = self.gql.mutate(mutations.update_lock, variables)
         lock = __.get(data, 'result')
         queue_index = __.arrays.find_index(
@@ -702,6 +706,38 @@ class ValiotWorker():
         # locked if last lock is active
         return __.get(current_queue, 'locks.0.active')
 
+    def setQueueLock(self, queue, lock, action):
+        """This function sets a lock to a queue.
+
+        Args:
+            queue_name (string): Queue's name
+            lock (object): Lock object
+            action (string): Action to do with the lock (one of 'created', 'updated', 'deleted')
+        """
+        queue_index = __.find_index(self.currentQueues, lambda q: q['name'] == queue['name'])
+        current_queue = __.get(self.currentQueues, queue_index)
+        if queue_index == -1:
+            return # no op, queue not found
+        match action:
+            case 'created':
+                self.log(LogLevel.INFO, f'Queue {queue["name"]} locked')
+                self.currentQueues[queue_index]['locks'] = [lock]
+            case 'updated':
+                self.log(LogLevel.INFO, f'Queue {queue["name"]} lock updated')
+                lock_index = __.find_index(current_queue['locks'], lambda l: l['id'] == lock['id'])
+                if lock_index == -1:
+                    self.currentQueues[queue_index]['locks'].append(lock)
+                    self.currentQueues[queue_index]['locks'] = __.sort_by(self.currentQueues[queue_index]['locks'], 'updatedAt', reverse=True)
+                else:
+                    self.currentQueues[queue_index]['locks'][lock_index] = lock
+            case 'deleted':
+                self.log(LogLevel.INFO, f'Queue {queue["name"]} lock released')
+                lock_index = __.find_index(current_queue['locks'], lambda l: l['id'] == lock['id'])
+                if lock_index != -1:
+                    self.currentQueues[queue_index]['locks'].pop(lock_index)
+            case _:
+                self.log(LogLevel.INFO, f'Queue {queue["name"]} unknown lock action: {action}')
+
     def workerQueueEnabled(self, queue):
         """This function checks if the queue is enable.
 
@@ -839,7 +875,7 @@ class ValiotWorker():
         # * first, get current configuration (may cause bugs if we create more than one config register, see Many-Many)
         workerQueues, errors = self.gql.query(
             query=queries.get_worker_queues,
-            variables={'workerCode': self.worker_code}
+            variables={'workerCode': f"^{self.worker['code']}$"}
         )
         if errors:
             self.log(LogLevel.ERROR, errors)
@@ -919,6 +955,7 @@ class ValiotWorker():
             'output': json.dumps({'data': output, 'errors': []})
         })
 
+    # TODO: unlock based on self.currentQueues['locks'] to avoid duplicate state and possible errors
     def unlockFinishedJobs(self, force=False):
         """This function updates all finished jobs to unlock them..
 
@@ -936,6 +973,7 @@ class ValiotWorker():
                 else:
                     _log(LogLevel.ERROR,
                          f'error unlocking {job_lock["job"]["queue"]["name"]}')
+                    unlocked.append(i) # remove from lockedJobs anyways to avoid infinite prints
         if unlocked:
             unlocked.reverse()
             for u in unlocked:
@@ -987,13 +1025,37 @@ class ValiotWorker():
             if (elapsedFromLastRun > freq):
                 # ! Run the frequency job!
                 if self.workerQueueEnabled(queue):
-                    self.gql.mutate(
-                        mutation=mutations.create_job,
-                        variables={
-                            'queueName': queue['name'],
-                            'worker': self.worker['code']
-                        }
-                    )
+                    if self.queue_locked(queue["name"]):
+                        self.log(LogLevel.ERROR,
+                                    f"{queue['name']['name']} is LOCKED, postponing...")
+                    else:
+                        self.gql.mutate(
+                            mutation=mutations.create_job,
+                            variables={
+                                'queueName': queue['name'],
+                                'worker': self.worker['code']
+                            }
+                        )
+
+    def registerLockChangesSubscription(self):
+        """Registers JobCreated subscription that will call certain job when activated"""
+        def onLockUpdated(data):
+            """Callback to run when locks are created/updated.
+            """
+            lock = __.get(data, 'result')
+            queue = __.get(lock, 'queue')
+            action = __.get(data, 'action')
+            if not lock or not __.get(data, 'successful'):
+                return  # Possibly a failed attempt to create lock
+            self.setQueueLock(queue, lock, action)
+
+        self.unsubLockUpdated = self.gql.subscribe(
+            query=subscriptions.LOCKS_UPDATED,
+            callback=onLockUpdated)
+        if (self.unsubLockUpdated):
+            self.log(LogLevel.SUCCESS, 'SUBSCRIBED TO LOCK CHANGES')
+        else:
+            self.log(LogLevel.ERROR, 'ERROR SUBSCRIBING TO LOCK CHANGES')
 
     def registerJobCreatedSubscription(self):
         """Registers JobCreated subscription that will call certain job when activated"""
@@ -1016,7 +1078,8 @@ class ValiotWorker():
 
         self.unsubJobCreated = self.gql.subscribe(
             query=subscriptions.JOB_CREATED,
-            variables={'workerCode': self.worker['code']},
+            # regex makes sure we only get jobs for this worker and not similar ones
+            variables={'workerCode': f"^{self.worker['code']}$"},
             callback=runOnDemandJob)
         if (self.unsubJobCreated):
             self.log(LogLevel.SUCCESS, 'SUBSCRIBED TO JOB CREATIONS')
@@ -1238,7 +1301,7 @@ class ValiotWorker():
                 self.continuousJobs.append(queue_name)
                 orderBy = self.queryOrderBy.value
                 queueLocks, errors = self.gql.query(queries.get_queueLocks, variables={
-                    "workerCode": self.worker_code,
+                    "workerCode": f"^{self.worker['code']}$",
                     "queueName": queue_name,
                     'orderByQueueLock': {'desc': orderBy}})
                 if errors:
@@ -1443,6 +1506,7 @@ class ValiotWorker():
                 LogLevel.INFO, 'No config sync performed. If required, set jobConfigMode to SYNC')
         if (self.pollingMode == PollingMode.SUBSCRIPTION):
             self.registerJobCreatedSubscription()
+        self.registerLockChangesSubscription()
         self.registerEventJobs()
         self.registerJobUpdatesListener()
         if self.useRedis:
